// XmlConfig.cpp: implementation of the CXmlConfig class.
//
//////////////////////////////////////////////////////////////////////

#include "XmlConfig.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CXmlConfig::CXmlConfig()
{

}

CXmlConfig::~CXmlConfig()
{

}
CXmlConfig::CXmlConfig(const char* chsFileName)
{
	LoadFile(chsFileName);
}

bool CXmlConfig::LoadFile(const char* chsFileName)
{
	if(!m_tXmlDoc.LoadFile(chsFileName)){
		printf( "Could not load '%s'. Error='%s'.\n", chsFileName, m_tXmlDoc.ErrorDesc() );
		return false;
	}
	m_pRootElem = m_tXmlDoc.RootElement();
	if(m_pRootElem == NULL)
	{
		return false;
	}
		
	return true;
}
unsigned int CXmlConfig::GetChildCount(TiXmlElement* pParent, const char* chsName)
{
	TiXmlElement* pelem = NULL;
	unsigned int nCount = 0;

	if(pParent == NULL)
	{
		return 0;
	}
	if(chsName != NULL)
	{
		pelem = pParent->FirstChildElement(chsName);
	}
	else
	{
		pelem = pParent->FirstChildElement();
	}
	if(!pelem)
		return nCount;

	nCount++;

    while(pelem != NULL)
	{
		if(chsName != NULL)
		{
			pelem = pelem->NextSiblingElement(chsName);
		}
		else
		{
			pelem = pelem->NextSiblingElement();
		}
		if(!pelem)
			break;

		nCount++;
    }

	return nCount;
}
TiXmlElement* CXmlConfig::GetChildElem(TiXmlElement* pParent, unsigned int nIndex, const char* chsName)
{
	TiXmlElement* pelem = NULL;
	unsigned int nCount = 0;
	
	if(pParent == NULL)
	{
		return NULL;
	}
	if(chsName != NULL)
	{
		pelem = pParent->FirstChildElement(chsName);
	}
	else
	{
		pelem = pParent->FirstChildElement();
	}

	if(!pelem)
		return NULL;

    while(pelem != NULL && nCount < nIndex)
	{
		if(chsName != NULL)
		{
			pelem = pelem->NextSiblingElement(chsName);
		}
		else
		{
			pelem = pelem->NextSiblingElement();
		}
		if(!pelem)
			break;

		nCount++;
    }
	
	if(nCount == nIndex)
		return pelem;

	return NULL;
}
bool CXmlConfig::GetAttrValueByName(TiXmlElement* pElem, const char* chsName, char* chsValue)
{
	if(chsName == NULL)
		return false;

	if(!pElem->Attribute(chsName))
	{
		return false;
	}
	strcpy(chsValue, pElem->Attribute(chsName));
	return true;
}
bool CXmlConfig::GetElemValueByName(TiXmlElement* pParent, const char* chsName, char* chsValue)
{
	TiXmlNode* pnode = NULL;
	TiXmlElement* pelem = NULL;

	if(pParent == NULL)
		return false;

	pelem = pParent->FirstChildElement(chsName);
	
	if(!pelem)
	{
		return false;
	}

	pnode = pelem->FirstChild();
	if(!pnode)
	{
		return false;
	}
	
	TiXmlText* ptext = pnode->ToText();
	if(!ptext)
	{
		return false;
	}
	
	strcpy(chsValue,ptext->Value());
	
	return true;
}

bool CXmlConfig::Parse(const char * str)
{
	m_tXmlDoc.Parse(str);
	if(m_tXmlDoc.Error()){
		printf("------Error Xml format!!!------\n");
		return false;
	}

	m_pRootElem = m_tXmlDoc.RootElement();
	if(m_pRootElem == NULL)
	{
		return false;
	}
		
	return true;
}
void CXmlConfig::GetXmlChar(char *value)
{
#ifdef TIXML_USE_STL
	ostrstream os;
	os << m_tXmlDoc;
	strcpy(value, os.str() );
#endif
}
const char *CXmlConfig::GetXmlChar()
{
#ifdef TIXML_USE_STL
	ostrstream os;
	os << m_tXmlDoc;
	return os.str();
#else
	return "";
#endif
}
void CXmlConfig::Print(void)
{
	m_tXmlDoc.Print(stdout);

}


