/***********************************************************
 * XmlPaser.cpp
 * TinyXmlLib
 *
 *  ܣ 
 *  Author: 
 *
 *  
 *  Created on: 2009-10-8
 *  
 ***********************************************************/

#include "XmlPaser.h"
#include "StringUtil.h"
using namespace std;


Element::Element(const Element& copy)
{
	this->m_xmlEle=copy.m_xmlEle;
}

Element& Element::operator=(const Element& base)
{
	//this->m_xmlEle.operator =(base.m))
	this->m_xmlEle.operator =(base.m_xmlEle);
	return *this;
}

void Element::SetAttribute(const char* pAttrName,const char* pAttrNameVal)
{
	this->m_xmlEle.SetAttribute(pAttrName,pAttrNameVal);
}

void Element::SetText(const char* pText)
{
	TiXmlText text(pText);
	this->m_xmlEle.InsertEndChild(text);
}

bool Element::AppendChild(const Element& child)
{
	return this->m_xmlEle.InsertEndChild(child.m_xmlEle)!=NULL;
}

void Element::print()
{
	this->m_xmlEle.Print(stdout,0);
}

const char* Element::GetAtrribute(const char* pAttrName)
{
	return m_xmlEle.Attribute(pAttrName);
}



XmlWriter::XmlWriter(const char* pRootName) {
	// TODO Auto-generated constructor stub
	this->m_xmlDoc=new TiXmlDocument();
	TiXmlElement t(pRootName);
	this->m_root=t;
	this->CreateEmptyXml();
	this->m_bIsSaved=false;
}
XmlWriter::XmlWriter(const char* pFileName,const char* pRootName)
{
	this->m_xmlDoc=new TiXmlDocument(pFileName);
	TiXmlElement t(pRootName);
	this->m_root=t;
	this->CreateEmptyXml();
	this->m_bIsSaved=false;
}
XmlWriter::~XmlWriter() {
	// TODO Auto-generated destructor stub
	if(this->m_xmlDoc)
		delete this->m_xmlDoc;
}

bool XmlWriter::CreateEmptyXml()
{
	//bool isSucess=false;
	TiXmlDeclaration dec("1.0","GB2312","no");
	if(!this->m_xmlDoc->InsertEndChild(dec))
		return false;
	return true;
	//return this->m_xmlDoc->InsertEndChild(this->m_root)==NULL;
	//this->m_root.InsertEndChild()
}

const char* XmlWriter::ToXmlStr(char* pXmlBuf)
{
	const char* pBuf=(const char*)pXmlBuf;
	Save();
	TiXmlPrinter printer;
	printer.SetIndent( "\t" );
	m_xmlDoc->Accept( &printer );
	strcpy(pXmlBuf,printer.CStr());
	return pBuf;
}

string XmlWriter::ToXmlStr()
{
	Save();
	TiXmlPrinter printer;
	printer.SetIndent( "\t" );
	m_xmlDoc->Accept( &printer );
	string xmlStr= printer.CStr();
	return xmlStr;
}


void XmlWriter::PrintXml()
{
	Save();
	this->m_xmlDoc->Print();
}

bool XmlWriter::SaveToFile(const char* pFileName)
{
	Save();
	return this->m_xmlDoc->SaveFile(pFileName);
}

bool XmlWriter::AppendElement(const Element* ele)
{
	if(NULL==ele)
		return false;
	checkSaved();
	return NULL!=this->m_root.InsertEndChild(ele->m_xmlEle);
}

void XmlWriter::checkSaved()
{
	if(this->m_bIsSaved)
	{
		this->m_xmlDoc->RemoveChild(&this->m_root);
		this->m_bIsSaved=false;
	}
}
void XmlWriter::Save()
{
	if(!this->m_bIsSaved){
		this->m_xmlDoc->InsertEndChild(this->m_root);
		this->m_bIsSaved=true;
	}
}

/////////////
const int XmlReader::FILE_READ_TYPE = 0;
const int XmlReader::STRING_READ_TYPE = 1;

XmlReader::XmlReader(const char* pName,int type)
{
	this->m_pName=const_cast<char*>(pName);
	this->m_nXmlReadType=type;
	this->m_xmlDoc=NULL;

	//this->m_docHandle=new TiXmlHandle(this->m_xmlDoc);
}

XmlReader::~XmlReader()
{
	if(this->m_xmlDoc)
		delete this->m_xmlDoc;
//	if(this->m_docHandle)
//		delete this->m_docHandle;
}

bool XmlReader::Parse()
{
	if(this->m_nXmlReadType==XmlReader::STRING_READ_TYPE){
		this->m_xmlDoc=new TiXmlDocument();
		return this->m_xmlDoc->Parse(this->m_pName)!=NULL;
	}
	else
	{
		this->m_xmlDoc=new TiXmlDocument(this->m_pName);
		bool bResult = this->m_xmlDoc->LoadFile(this->m_pName);
		//cout<<this->m_xmlDoc->ErrorDesc()<<endl;;
		//this->m_xmlDoc->Parse(pName);
		return bResult;
	}
}


/**
 * pElepath/ͷ
 *
 */
int XmlReader::GetFirstTextValue(const char* pElePath,char* pTxtVal)
{
	if(NULL==this->m_xmlDoc)
		return -2;
	TiXmlElement* ele=QuerySingleHandle(pElePath,0);

	if(NULL==ele)
	{
		return -1;
	}
	//cout<<ele->GetText()<<endl;
	if(NULL==ele->GetText())
	{
		return -2;
	}
	strcpy(pTxtVal,ele->GetText());
	return 0;
}

int XmlReader::GetTextValue(const char* pElePath,char* pTxtVal,int nIndex)
{
	if(NULL==this->m_xmlDoc)
		return -2;
	TiXmlElement* ele=QuerySingleHandle(pElePath,nIndex);

	if(NULL==ele)
	{
		return -1;
	}
	if(NULL==ele->GetText())
	{
		return -2;
	}
	strcpy(pTxtVal,ele->GetText());
	return 0;
}

int XmlReader::GetAttribute(const char* pElePath,const char* pAttrName,char* pAttrVal)
{
	if(NULL==this->m_xmlDoc)
		return -2;
	TiXmlElement* ele=QuerySingleHandle(pElePath,0);

	if(NULL==ele)
	{
		return -1;
	}
	if(!ele->Attribute(pAttrName))
		return -2;
	strcpy(pAttrVal,ele->Attribute(pAttrName));
	return 0;
}

int XmlReader::GetElements(const char* pElePath,vector<Element*>& eles)
{
	if(NULL==this->m_xmlDoc)
		return -2;
	TiXmlElement* ele=QuerySingleHandle(pElePath,0);
	if(ele==NULL)
	{
		return -1;
	}
//	if(handler==NULL)
//		return -2;
//	TiXmlElement* child=handler.ToElement();
	Element* element=NULL;
	for(ele;ele!=0;ele=ele->NextSiblingElement())
	{

		element=new Element();
		//cout<<"========>GetElements before========"<<endl;
		element->m_xmlEle=*ele;
		//cout<<ele->GetText()<<endl;
		//cout<<"========>GetElements end========"<<endl;
		eles.push_back(element);
		element=NULL;
	}
	return 0;
}

int XmlReader::GetSubElements(const char* pElePath,vector<Element*>& eles)
{
	if(NULL==this->m_xmlDoc)
		return -2;
	TiXmlElement* ele=QuerySingleHandle(pElePath,0);
	if(ele==NULL)
	{
		return -1;
	}
//	if(handler==NULL)
//		return -2;
//	TiXmlElement* child=handler.ToElement();
	Element* element=NULL;
	TiXmlElement* e=NULL;
	for(e=ele->FirstChildElement();e;e=e->NextSiblingElement())
	{
		element=new Element();
		element->m_xmlEle=*e;
		eles.push_back(element);
		element=NULL;
	}
	return 0;
}

int XmlReader::GetSubElements(const char* pElePath,vector<Element>& eles)
{
	if(NULL==this->m_xmlDoc)
			return -2;
	TiXmlElement* ele=QuerySingleHandle(pElePath,0);
	if(ele==NULL)
	{
		return -1;
	}
	TiXmlElement* e=NULL;
	for(e=ele->FirstChildElement();e;e=e->NextSiblingElement())
	{
		Element ele;
		ele.m_xmlEle=*e;
		eles.push_back(ele);
	}
	return 0;
}

int XmlReader::DestoryElements(vector<Element*>&eles)
{
	if(NULL==this->m_xmlDoc)
		return -2;
	for(vector<Element*>::iterator iter=eles.begin();iter!=eles.end();)
	{
		Element* pEle=*iter;
		if(NULL==pEle)
		{
			eles.erase(iter);
		}else{
			delete pEle;
			iter++;
		}

	}
	eles.clear();
	return 0;
}

int XmlReader::GetRootValue(char* pVal)
{
	if(NULL==this->m_xmlDoc)
		return -2;
	//cout<<"==>xml::"<<this->m_xmlDoc->RootElement()<<endl;
	 strcpy(pVal,this->m_xmlDoc->RootElement()->Value());
	 return strlen(pVal);
}

Element* XmlReader::GetRoot()
{
	if(NULL==this->m_xmlDoc)
		return NULL;
	Element* ele=new Element();
	ele->m_xmlEle=*(this->m_xmlDoc->RootElement());
	return ele;
}

TiXmlElement* XmlReader::QuerySingleHandle(const char* pElePath,int nIndex)
{
	if(pElePath==NULL||strlen(pElePath)<1)
	{
#ifdef __DEBUG__
		cout<<"xptah is NULL or length is 0"<<endl;
#endif
		return NULL;
	}
	string xPath=pElePath;
	if(pElePath[0]!='/')
	{
		xPath='/'+xPath;
		//return -2;
	}

	deque<string> vals;

	CStringUtil::StrSplit(xPath,vals,'/');
	int nLen=vals.size();
	TiXmlElement* element=this->m_xmlDoc->FirstChildElement(vals.at(0).c_str());
	if(!element)
		return NULL;
	for(int i=1;i<nLen;i++)
	{
		element=element->FirstChildElement(vals.at(i).c_str());
		if(NULL==element)
			return NULL;
	}
	if(nIndex<=0)
	{
		return element;
	}
	int count=1;

	TiXmlElement* e=NULL;
	while((e=element->NextSiblingElement(vals.at(nLen-1).c_str()))!=NULL)
	{
		if(count==nIndex)
			return e;
		count++;
	}
	return NULL;
}

