package com.wisentsoft.isms;

import com.google.gson.Gson;
import org.apache.commons.codec.CharEncoding;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.conn.ssl.NoopHostnameVerifier;
import org.apache.http.conn.ssl.SSLConnectionSocketFactory;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.ssl.SSLContextBuilder;
import org.apache.http.ssl.TrustStrategy;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.net.ssl.SSLContext;
import java.io.IOException;
import java.security.KeyManagementException;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.List;
import java.util.Locale;

/**
 * Hello world!
 */
public class App {
    private static final String recvApi = "https://dx.11185.cn:13012/isms-recv-api";

    private static final Logger log = LoggerFactory.getLogger(App.class);

    private static final Gson gson = new Gson();

    private static final String appId = "appKey";
    private static final String secret = "密钥";

    public static void main(String[] args) {
        System.out.println("Hello World!");

        for (; ; ) {
            List<SmsUpItem> smsUpItemList = getMoList(appId, secret);
            for (SmsUpItem item : smsUpItemList) {
                // 处理代码 ...
            }
            try {
                Thread.sleep(200);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
    }

    private static List<SmsUpItem> getMoList(String appId, String secret) {

        String url = String.format(Locale.CHINA, "%s/recv/smsUp", recvApi);

        HttpPost httppost = new HttpPost(url);

        httppost.setHeader("appId", appId);
        httppost.setHeader("secret", secret);

        CloseableHttpClient httpClient = createHttpClient();
        if (null == httpClient) {
            return null;
        }

        CloseableHttpResponse response = null;
        try {
            response = httpClient.execute(httppost);
            HttpEntity entity = response.getEntity();
            if (null == entity) {
                log.error("调用上行接口失败");
                return null;
            }

            String result = EntityUtils.toString(entity, CharEncoding.UTF_8);
            log.info("result - " + result);
            if (StringUtils.isBlank(result)) {
                log.error("调用上行接口返回结果为空");
                return null;
            }
            MoErrorResult moErrorResult = gson.fromJson(result, MoErrorResult.class);

            if (null == moErrorResult) {
                log.error("调用上行接口返回结果无法解析");
                return null;
            }
            if (MoErrorResult.ERROR_CODE_OK != moErrorResult.getErrorCode()) {
                log.error("调用上行接口返回报错: {}", moErrorResult.getErrorCode());
                return null;
            }

            return moErrorResult.getData();
        } catch (Exception e) {
            log.error("调用发送接口出现异常", e);
            return null;
        } finally {
            if (null != response) {
                try {
                    response.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    private static CloseableHttpClient createHttpClient() {
        try {
            SSLContext sslContext = new SSLContextBuilder().loadTrustMaterial(null, new TrustStrategy() {
                @Override
                public boolean isTrusted(X509Certificate[] chain, String authType) throws CertificateException {
                    return true;
                }
            }).build();
            SSLConnectionSocketFactory sslConnectionSocketFactory = new SSLConnectionSocketFactory(sslContext, NoopHostnameVerifier.INSTANCE);
            return HttpClients.custom().setSSLSocketFactory(sslConnectionSocketFactory).build();
        } catch (NoSuchAlgorithmException | KeyStoreException | KeyManagementException e) {
            log.error("创建 httpClient fail", e);
            return null;
        }

    }
}
